from typing import List, Type, Tuple, TypeVar

from aixplain.enums import DataType

from .base import Serializable, Node, Link
from .nodes import (
    AssetNode,
    Decision,
    Script,
    Input,
    Output,
    Router,
    Route,
    BareReconstructor,
    BareSegmentor,
)
from .enums import NodeType, RouteType, Operation


T = TypeVar("T", bound="AssetNode")


class DesignerPipeline(Serializable):
    nodes: List[Node] = None
    links: List[Link] = None
    instance: any = None

    def __init__(self):
        self.nodes = []
        self.links = []

    def add_node(self, node: Node):
        """
        Add a node to the current pipeline.

        This method will take care of setting the pipeline instance to the
        node and setting the node number if it's not set.

        :param node: the node
        :return: the node
        """
        return node.attach_to(self)

    def add_nodes(self, *nodes: Node) -> List[Node]:
        """
        Add multiple nodes to the current pipeline.

        :param nodes: the nodes
        :return: the nodes
        """
        return [self.add_node(node) for node in nodes]

    def add_link(self, link: Link) -> Link:
        """
        Add a link to the current pipeline.
        :param link: the link
        :return: the link
        """
        return link.attach_to(self)

    def serialize(self) -> dict:
        """
        Serialize the pipeline to a dictionary. This method will serialize the
        pipeline to a dictionary.

        :return: the pipeline as a dictionary
        """
        return {
            "nodes": [node.serialize() for node in self.nodes],
            "links": [link.serialize() for link in self.links],
        }

    def validate_nodes(self):
        """
        Validate the linkage of the pipeline. This method will validate the
        linkage of the pipeline by applying the following checks:
        - All input nodes are linked out
        - All output nodes are linked in
        - All other nodes are linked in and out

        :raises ValueError: if the pipeline is not valid
        """
        link_from_map = {link.from_node.number: link for link in self.links}
        link_to_map = {link.to_node.number: link for link in self.links}
        contains_input = False
        contains_output = False
        contains_asset = False
        for node in self.nodes:
            # validate every input node is linked out
            if node.type == NodeType.INPUT:
                contains_input = True
                if node.number not in link_from_map:
                    raise ValueError(f"Input node {node.label} not linked out")
            # validate every output node is linked in
            elif node.type == NodeType.OUTPUT:
                contains_output = True
                if node.number not in link_to_map:
                    raise ValueError(f"Output node {node.label} not linked in")
            # validate rest of the nodes are linked in and out
            else:
                if isinstance(node, AssetNode):
                    contains_asset = True
                if node.number not in link_from_map:
                    raise ValueError(f"Node {node.label} not linked in")
                if node.number not in link_to_map:
                    raise ValueError(f"Node {node.label} not linked out")

        if not contains_input or not contains_output or not contains_asset:
            raise ValueError(
                "Pipeline must contain at least one input, output and asset node"  # noqa
            )

    def is_param_linked(self, node, param):
        """
        Check if the param is linked to another node. This method will check
        if the param is linked to another node.
        :param node: the node
        :param param: the param
        :return: True if the param is linked, False otherwise
        """
        for link in self.links:
            if (
                link.to_node.number == node.number
                and param.code == link.to_param
            ):
                return True

        return False

    def is_param_set(self, node, param):
        """
        Check if the param is set. This method will check if the param is set
        or linked to another node.
        :param node: the node
        :param param: the param
        :return: True if the param is set, False otherwise
        """
        return param.value or self.is_param_linked(node, param)

    def validate_params(self):
        """
        This method will check if all required params are either set or linked

        :raises ValueError: if the pipeline is not valid
        """
        for node in self.nodes:
            for param in node.inputs:
                if param.is_required and not self.is_param_set(node, param):
                    raise ValueError(
                        f"Param {param.code} of node {node.label} is required"
                    )

    def validate(self):
        """
        Validate the pipeline. This method will validate the pipeline by
        series of checks:
        - Validate all nodes are linked correctly
        - Validate all required params are set or linked

        Any other validation checks can be added here.

        :raises ValueError: if the pipeline is not valid
        """
        self.validate_nodes()
        self.validate_params()

    def get_link(self, from_node: int, to_node: int) -> Link:
        """
        Get the link between two nodes. This method will return the link
        between two nodes.

        :param from_node: the from node number
        :param to_node: the to node number
        :return: the link
        """
        return next(
            (
                link
                for link in self.links
                if link.from_node == from_node and link.to_node == to_node
            ),
            None,
        )

    def get_node(self, node_number: int) -> Node:
        """
        Get the node by its number. This method will return the node with the
        given number.

        :param node_number: the node number
        :return: the node
        """
        return next(
            (node for node in self.nodes if node.number == node_number), None
        )

    def auto_infer(self):
        """
        Automatically infer the data types of the nodes in the pipeline.
        This method will automatically infer the data types of the nodes in the
        pipeline by traversing the pipeline and setting the data types of the
        nodes based on the data types of the connected nodes.
        """
        for link in self.links:
            from_node = self.get_node(link.from_node)
            to_node = self.get_node(link.to_node)
            if not from_node or not to_node:
                continue  # will be handled by the validation
            for param in link.param_mapping:
                from_param = from_node.outputs[param.from_param]
                to_param = to_node.inputs[param.to_param]
                if not from_param or not to_param:
                    continue  # will be handled by the validation
                # if one of the data types is missing, infer the other one
                dataType = from_param.data_type or to_param.data_type
                from_param.data_type = dataType
                to_param.data_type = dataType

            def infer_data_type(node):
                from .nodes import Input, Output

                if isinstance(node, Input) or isinstance(node, Output):
                    if dataType and dataType not in node.data_types:
                        node.data_types.append(dataType)

            infer_data_type(self)
            infer_data_type(to_node)

    def asset(
        self, asset_id: str, *args, asset_class: Type[T] = AssetNode, **kwargs
    ) -> T:
        """
        Shortcut to create an asset node for the current pipeline.
        All params will be passed as keyword arguments to the node
        constructor.

        :param kwargs: keyword arguments
        :return: the node
        """
        return asset_class(asset_id, *args, pipeline=self, **kwargs)

    def decision(self, *args, **kwargs) -> Decision:
        """
        Shortcut to create an decision node for the current pipeline.
        All params will be passed as keyword arguments to the node
        constructor.

        :param kwargs: keyword arguments
        :return: the node
        """
        return Decision(*args, pipeline=self, **kwargs)

    def script(self, *args, **kwargs) -> Script:
        """
        Shortcut to create an script node for the current pipeline.
        All params will be passed as keyword arguments to the node
        constructor.

        :param kwargs: keyword arguments
        :return: the node
        """
        return Script(*args, pipeline=self, **kwargs)

    def input(self, *args, **kwargs) -> Input:
        """
        Shortcut to create an input node for the current pipeline.
        All params will be passed as keyword arguments to the node
        constructor.

        :param kwargs: keyword arguments
        :return: the node
        """
        return Input(*args, pipeline=self, **kwargs)

    def output(self, *args, **kwargs) -> Output:
        """
        Shortcut to create an output node for the current pipeline.
        All params will be passed as keyword arguments to the node
        constructor.

        :param kwargs: keyword arguments
        :return: the node
        """
        return Output(*args, pipeline=self, **kwargs)

    def router(self, routes: Tuple[DataType, Node], *args, **kwargs) -> Router:
        """
        Shortcut to create an decision node for the current pipeline.
        All params will be passed as keyword arguments to the node
        constructor. The routes will be handled specially and will be
        converted to Route instances in a convenient way.

        :param routes: the routes
        :param kwargs: keyword arguments
        :return: the node
        """
        kwargs["routes"] = [
            Route(
                value=route[0],
                path=[route[1]],
                type=RouteType.CHECK_TYPE,
                operation=Operation.EQUAL,
            )
            for route in routes
        ]
        return Router(*args, pipeline=self, **kwargs)

    def bare_reconstructor(self, *args, **kwargs) -> BareReconstructor:
        """
        Shortcut to create an reconstructor node for the current pipeline.
        All params will be passed as keyword arguments to the node
        constructor.

        :param kwargs: keyword arguments
        :return: the node
        """
        return BareReconstructor(*args, pipeline=self, **kwargs)

    def bare_segmentor(self, *args, **kwargs) -> BareSegmentor:
        """
        Shortcut to create an segmentor node for the current pipeline.
        All params will be passed as keyword arguments to the node
        constructor.

        :param kwargs: keyword arguments
        :return: the node
        """
        return BareSegmentor(*args, pipeline=self, **kwargs)



from aixplain.utils.file_utils import _request_with_retry
from urllib.parse import urljoin
import logging
from aixplain.utils import config

def delete_asset(model_id, api_key):
    delete_url = urljoin(config.BACKEND_URL, f"sdk/models/{model_id}")
    logging.debug(f"URL: {delete_url}")
    headers = {"Authorization": f"Token {api_key}", "Content-Type": "application/json"}
    _ = _request_with_retry("delete", delete_url, headers=headers)

def delete_service_account(api_key):
    delete_url = urljoin(config.BACKEND_URL, f"sdk/ecr/logout")
    logging.debug(f"URL: {delete_url}")
    headers = {"Authorization": f"Token {api_key}", "Content-Type": "application/json"}
    _ = _request_with_retry("post", delete_url, headers=headers)


from requests.adapters import HTTPAdapter, Retry
import requests
from typing import Text


def _request_with_retry(method: Text, url: Text, **params) -> requests.Response:
    """Wrapper around requests with Session to retry in case it fails

    Args:
        method (Text): HTTP method, such as 'GET' or 'HEAD'.
        url (Text): The URL of the resource to fetch.
        **params: Params to pass to request function

    Returns:
        requests.Response: Response object of the request
    """
    session = requests.Session()
    retries = Retry(total=5, backoff_factor=0.1, status_forcelist=[500, 502, 503, 504])
    session.mount("https://", HTTPAdapter(max_retries=retries))
    response = session.request(method=method.upper(), url=url, **params)
    return response



__author__ = "lucaspavanelli"

"""
Copyright 2022 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: Duraikrishna Selvaraju, Thiago Castro Ferreira, Shreyas Sharma and Lucas Pavanelli
Date: June 14th 2023
Description:
    Finetune Factory Class
"""

import logging
from typing import Dict, List, Optional, Text, Union
import json
from aixplain.factories.dataset_factory import DatasetFactory
from aixplain.factories.finetune_factory.prompt_validator import validate_prompt
from aixplain.factories.model_factory import ModelFactory
from aixplain.modules.finetune import Finetune
from aixplain.modules.finetune.cost import FinetuneCost
from aixplain.modules.finetune.hyperparameters import Hyperparameters
from aixplain.modules.dataset import Dataset
from aixplain.modules.model import Model
from aixplain.utils import config
from aixplain.utils.file_utils import _request_with_retry
from urllib.parse import urljoin


class FinetuneFactory:
    """A static class for creating and managing the FineTune experience.

    Attributes:
        backend_url (str): The URL for the backend.
    """

    aixplain_key = config.AIXPLAIN_API_KEY
    backend_url = config.BACKEND_URL

    @classmethod
    def _create_cost_from_response(cls, response: Dict) -> FinetuneCost:
        """Create a Cost object from the response dictionary.

        Args:
            response (Dict): The response dictionary containing cost information.

        Returns:
            Cost: The Cost object created from the response.
        """
        return FinetuneCost(response["trainingCost"], response["inferenceCost"], response["hostingCost"])

    @classmethod
    def create(
        cls,
        name: Text,
        dataset_list: List[Union[Dataset, Text]],
        model: Union[Model, Text],
        prompt_template: Optional[Text] = None,
        hyperparameters: Optional[Hyperparameters] = None,
        train_percentage: Optional[float] = 100,
        dev_percentage: Optional[float] = 0,
    ) -> Finetune:
        """Create a Finetune object with the provided information.

        Args:
            name (Text): Name of the Finetune.
            dataset_list (List[Dataset]): List of Datasets (or dataset IDs) to be used for fine-tuning.
            model (Model): Model (Model ID) to be fine-tuned.
            prompt_template (Text, optional): Fine-tuning prompt_template. Should reference columns in the dataset using format <<COLUMN_NAME>>. Defaults to None.
            hyperparameters (Hyperparameters, optional): Hyperparameters for fine-tuning. Defaults to None.
            train_percentage (float, optional): Percentage of training samples. Defaults to 100.
            dev_percentage (float, optional): Percentage of development samples. Defaults to 0.
        Returns:
            Finetune: The Finetune object created with the provided information or None if there was an error.
        """
        payload = {}
        assert train_percentage > 0, f"Create FineTune: Train percentage ({train_percentage}) must be greater than zero"
        assert (
            train_percentage + dev_percentage <= 100
        ), f"Create FineTune: Train percentage + dev percentage ({train_percentage + dev_percentage}) must be less than or equal to one"
        
        for i, dataset in enumerate(dataset_list):
            if isinstance(dataset, str) is True:
                dataset_list[i] = DatasetFactory.get(dataset_id=dataset)
        
        if isinstance(model, str) is True:
            model = ModelFactory.get(model_id=model)

        if prompt_template is not None:
            prompt_template = validate_prompt(prompt_template, dataset_list)
        try:
            url = urljoin(cls.backend_url, f"sdk/finetune/cost-estimation")
            headers = {"Authorization": f"Token {config.TEAM_API_KEY}", "Content-Type": "application/json"}
            payload = {
                "datasets": [
                    {"datasetId": dataset.id, "trainPercentage": train_percentage, "devPercentage": dev_percentage}
                    for dataset in dataset_list
                ],
                "sourceModelId": model.id,
            }
            parameters = {}
            if prompt_template is not None:
                parameters["prompt"] = prompt_template
            if hyperparameters is not None:
                parameters["hyperparameters"] = hyperparameters.to_dict()
            payload["parameters"] = parameters
            logging.info(f"Start service for POST Create FineTune - {url} - {headers} - {json.dumps(payload)}")
            r = _request_with_retry("post", url, headers=headers, json=payload)
            resp = r.json()
            logging.info(f"Response for POST Create FineTune - Status {resp}")
            cost = cls._create_cost_from_response(resp)
            return Finetune(
                name,
                dataset_list,
                model,
                cost,
                train_percentage=train_percentage,
                dev_percentage=dev_percentage,
                prompt_template=prompt_template,
                hyperparameters=hyperparameters,
            )
        except Exception:
            error_message = f"Create FineTune: Error with payload {json.dumps(payload)}"
            logging.exception(error_message)
            return None



__author__ = "shreyassharma"

"""
Copyright 2023 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: aiXplain team
Date: March 20th 2023
Description:
    File Factory Class
"""

import os
import validators
import filetype
from aixplain.enums.storage_type import StorageType
from aixplain.enums.license import License
from aixplain.utils.file_utils import upload_data
from typing import Any, Dict, Text, Union, Optional, List

MB_1 = 1048576
MB_25 = 26214400
MB_50 = 52428800
MB_300 = 314572800


class FileFactory:
    @classmethod
    def upload(
        cls, local_path: Text, tags: Optional[List[Text]] = None, license: Optional[License] = None, is_temp: bool = True
    ) -> Text:
        """
        Uploads a file to an S3 bucket.

        Args:
            local_path (Text): The local path of the file to upload.
            tags (List[Text], optional): tags of the file
            license (License, optional): the license for the file
            is_temp (bool): specify if the file that will be upload is a temporary file

        Returns:
            Text: The S3 path where the file was uploaded.

        Raises:
            FileNotFoundError: If the local file is not found.
            Exception: If the file size exceeds the maximum allowed size.
        """
        if os.path.exists(local_path) is False:
            raise FileNotFoundError(f'File Upload Error: local file "{local_path}" not found.')
        # mime type format: {type}/{extension}
        mime_type = filetype.guess_mime(local_path)
        if mime_type is None:
            content_type = "text/csv"
        else:
            content_type = mime_type

        type_to_max_size = {"audio": MB_50, "application": MB_25, "video": MB_300, "image": MB_25, "other": MB_50}
        if mime_type is None or mime_type.split("/")[0] not in type_to_max_size:
            ftype = "other"
        else:
            ftype = mime_type.split("/")[0]
        if os.path.getsize(local_path) > type_to_max_size[ftype]:
            raise Exception(
                f'File Upload Error: local file "{local_path}" of type "{mime_type}" exceeds {type_to_max_size[ftype] / MB_1} MB.'
            )

        if is_temp is False:
            s3_path = upload_data(file_name=local_path, tags=tags, license=license, is_temp=is_temp, content_type=content_type)
        else:
            s3_path = upload_data(file_name=local_path)
        return s3_path

    @classmethod
    def check_storage_type(cls, input_link: Any) -> StorageType:
        """Check whether a path is a URL (s3 link or HTTP link), a file or a textual content

        Args:
            input_link (Any): path to be checked

        Returns:
            StorageType: URL, TEXT or FILE
        """
        if os.path.exists(input_link) is True:
            return StorageType.FILE
        elif (
            input_link.startswith("s3://")
            or input_link.startswith("http://")
            or input_link.startswith("https://")
            or validators.url(input_link)
        ):
            return StorageType.URL
        else:
            return StorageType.TEXT

    @classmethod
    def to_link(cls, data: Union[Text, Dict], **kwargs) -> Union[Text, Dict]:
        """If user input data is a local file, upload to aiXplain platform

        Args:
            data (Union[Text, Dict]): input data

        Returns:
            Union[Text, Dict]: input links/texts
        """
        if isinstance(data, dict):
            for key in data:
                if isinstance(data[key], str):
                    if cls.check_storage_type(data[key]) == StorageType.FILE:
                        data[key] = cls.upload(local_path=data[key], **kwargs)
        elif isinstance(data, str):
            if cls.check_storage_type(data) == StorageType.FILE:
                data = cls.upload(local_path=data, **kwargs)
        return data

    @classmethod
    def create(
        cls, local_path: Text, tags: Optional[List[Text]] = None, license: Optional[License] = None, is_temp: bool = False
    ) -> Text:
        """
        Uploads a file to an S3 bucket.

        Args:
            local_path (Text): The local path of the file to upload.
            tags (List[Text], optional): tags of the file
            license (License, optional): the license for the file
            is_temp (bool): specify if the file that will be upload is a temporary file

        Returns:
            Text: The S3 path where the file was uploaded.

        Raises:
            FileNotFoundError: If the local file is not found.
            Exception: If the file size exceeds the maximum allowed size.
        """
        assert (
            license is not None if is_temp is False else True
        ),  "File Asset Creation Error: To upload a non-temporary file, you need to specify the `license`."
        return cls.upload(local_path=local_path, tags=tags, license=license, is_temp=is_temp)



__author__ = "thiagocastroferreira"

"""
Copyright 2022 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import pytest
import time
from aixplain.enums import Language, License, OnboardStatus
from aixplain.factories.corpus_factory import CorpusFactory
from uuid import uuid4


def test_corpus_onboard_get_delete():
    upload_file = "tests/functional/data_asset/input/audio-en_url.csv"
    schema = [
        {
            "name": "audio",
            "dtype": "audio",
            "storage_type": "url",
            "start_column": "audio_start_time",
            "end_column": "audio_end_time",
            "languages": [Language.English_UNITED_STATES],
        },
        {"name": "text", "dtype": "text", "storage_type": "text", "languages": [Language.English_UNITED_STATES]},
    ]

    response = CorpusFactory.create(
        name=str(uuid4()),
        description="This corpus contain 20 English audios with their corresponding transcriptions.",
        license=License.MIT,
        content_path=upload_file,
        schema=schema,
    )
    asset_id = response["asset_id"]
    onboard_status = OnboardStatus(response["status"])
    while onboard_status == OnboardStatus.ONBOARDING:
        corpus = CorpusFactory.get(asset_id)
        onboard_status = corpus.onboard_status
        time.sleep(1)
    # assert the asset was onboarded
    assert onboard_status == OnboardStatus.ONBOARDED
    # assert the asset was deleted
    corpus.delete()
    with pytest.raises(Exception):
        corpus = CorpusFactory.get(asset_id)


def test_corpus_listing():
    response = CorpusFactory.list()
    assert "results" in response


def test_corpus_get_error():
    with pytest.raises(Exception):
        response = CorpusFactory.get("131312")



"""
Copyright 2022 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from typing import Union, Dict, List 
from aixplain.modules.metadata import MetaData


def dict_to_metadata(metadatas: List[Union[Dict, MetaData]]) -> None:
    
    """Convert all the Dicts to MetaData

    Args:
        metadatas (List[Union[Dict, MetaData]], optional): metadata of metadata information of the dataset.

    """
    try:
        for i in range(len(metadatas)):
            if isinstance(metadatas[i], dict):
                metadatas[i] = MetaData(**metadatas[i])
    except TypeError:
        raise TypeError(f"Data Asset Onboarding Error: One or more elements in the metadata_schema are not well-structured")
        
    


# Copyright 2023 aiXplain authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The Python implementation of the aiXplain speech recoginition streaming client."""

import argparse
import time
from typing import List
import grpc
import logging

import aixplain_diarization_streaming_pb2 as pb
import aixplain_diarization_streaming_pb2_grpc

FRAME_RATE = 16000

def generate_payloads(file_path, latency):
    stream_configuration = pb.DiarizationRequest(
        config=pb.AudioConfig(encoding="LINEAR16", hertz=FRAME_RATE, language_code="en"),
        latency=latency,
        diarization_config=pb.DiarizationConfig(min_speaker_count=1, max_speaker_count=3),
    )
    yield stream_configuration
    # Iterate over the raw bytes in chunks
    chunk_size = 16000 # half a second of audio
    i = 0
    with open(file_path, "rb") as audio_file:
        while True:
            chunk = audio_file.read(chunk_size)
            if not chunk:
                break
            logging.info(f'Sending chunk {i}')
            payload = pb.DiarizationRequest(audio_content=chunk)
            yield payload
            i += 1
            time.sleep(0.5) # simulate streaming by introducing sleep

def grpc_duration_to_seconds(duration):
    seconds = float(duration.seconds)
    nanos = float(duration.nanos) / 1e9
    return seconds + nanos

def consume_results(response: List[pb.DiarizationResponse]):
    for inference in response:
        if inference.is_final:
            logging.info(f'Received is_final={inference.is_final}. total_time={inference.end_time.seconds}.{str(inference.end_time.nanos)[:3]}')
        if len(inference.segments):
            logging.info(f'Turns:')
            for segment in inference.segments:
                logging.info(f"{segment.speaker_tag} \
                    start:{grpc_duration_to_seconds(segment.start_time)}\tend:{grpc_duration_to_seconds(segment.end_time)}")

def _stream_file(channel, file_path, latency):
    stub = aixplain_diarization_streaming_pb2_grpc.AixplainDiarizationStreamingStub(channel)
    response = stub.Diarize(generate_payloads(file_path, latency))
    consume_results(response)

def run_insecure(host, file_path, latency):
    with grpc.insecure_channel(host, options=(('grpc.ssl_target_name_override', host),)) as channel:
        _stream_file(channel, file_path, latency)

def run(host, client_cert, client_key, ca_cert, file_path, latency):
    def create_secure_channel(host, client_cert, client_key, ca_cert):
        with open(client_cert, 'rb') as f:
            client_cert_data = f.read()

        with open(client_key, 'rb') as f:
            client_key_data = f.read()

        with open(ca_cert, 'rb') as f:
            ca_cert_data = f.read()

        credentials = grpc.ssl_channel_credentials(
            root_certificates=ca_cert_data,
            private_key=client_key_data,
            certificate_chain=client_cert_data
        )
        return grpc.secure_channel(host, credentials)
    with create_secure_channel(host, client_cert, client_key, ca_cert) as channel:
        _stream_file(channel, file_path, latency)


if __name__ == "__main__":
    log_format = "%(asctime)s - %(levelname)-7s - P%(process)d: %(message)s"
    logging.basicConfig(level=logging.INFO, format=log_format)
    parser = argparse.ArgumentParser(description="aiXplain speech recognition streaming client.")

    parser.add_argument('--addr', default='localhost:50051', help='the address to connect to (default "localhost:50051")')
    parser.add_argument('--cacert', default='./client-crt/ca.crt', help='ca cert for mTLS (default "./client-crt/ca.crt")')
    parser.add_argument('--cert', default='./client-crt/tls.crt', help='client cert for mTLS (default "./client-crt/tls.crt")')
    parser.add_argument('--key', default='./client-crt/tls.key', help='client key for mTLS (default "./client-crt/tls.key")')
    parser.add_argument('--insecure', action='store_true', help='use insecure connection (no mTLS)')
    parser.add_argument('--file-path', help='audio file to stream from')
    parser.add_argument('--latency', type=float, help='Model latency')

    args = parser.parse_args()

    if args.insecure:
        run_insecure(args.addr, args.file_path, args.latency)
    else:
        run(args.addr, args.cert, args.key, args.cacert, args.file_path, args.latency)



__author__ = "thiagocastroferreira"

import logging
import os
import pandas as pd
import validators

from aixplain.enums.file_type import FileType
from aixplain.enums.storage_type import StorageType
from aixplain.modules.file import File
from aixplain.modules.metadata import MetaData
from aixplain.utils.file_utils import upload_data
from pathlib import Path
from tqdm import tqdm
from typing import List, Optional, Text, Tuple


def process_text(content: str, storage_type: StorageType) -> Text:
    """Process text files

    Args:
        content (str): URL with text, local path with text or textual content
        storage_type (StorageType): type of storage: URL, local path or textual content

    Returns:
        Text: textual content
    """
    if storage_type == StorageType.FILE:
        # Check the size of file and assert a limit of 25 MB
        assert (
            os.path.getsize(content) <= 25000000
        ), f'Data Asset Onboarding Error: Local text file "{content}" exceeds the size limit of 25 MB.'
        with open(content) as f:
            text = f.read()
    else:
        text = content

    # if the row is a textual URL (which should not be downloaded), tag it
    if storage_type in [StorageType.FILE, StorageType.TEXT] and (
        str(text).startswith("s3://")
        or str(text).startswith("http://")
        or str(text).startswith("https://")
        or validators.url(text)
    ):
        text = "DONOTDOWNLOAD" + str(text)
    return text


def run(metadata: MetaData, paths: List, folder: Path, batch_size: int = 1000) -> Tuple[List[File], int, int]:
    """Process a list of local textual files, compress and upload them to pre-signed URLs in S3

    Explanation:
        Each text on "paths" is processed. If the text is in a public link or local file, it will be downloaded and added to an index CSV file.
        The texts are processed in batches such that at each "batch_size" texts, the index CSV file is uploaded into a pre-signed URL in s3 and reset.

    Args:
        metadata (MetaData): meta data of the asset
        paths (List): list of paths to local files
        folder (Path): local folder to save compressed files before upload them to s3.

    Returns:
        Tuple[List[File], int, int]: list of s3 links, data colum index and number of rows
    """
    logging.debug(f'Data Asset Onboarding: Processing "{metadata.name}".')
    idx = 0
    data_column_idx = -1
    files, batch = [], []
    for i in tqdm(range(len(paths)), desc=f' Data "{metadata.name}" onboarding progress', position=1, leave=False):
        path = paths[i]
        try:
            dataframe = pd.read_csv(path)
        except Exception as e:
            message = f'Data Asset Onboarding Error: Local file "{path}" not found.'
            logging.exception(message)
            raise Exception(message)

        # process texts and labels
        for j in tqdm(range(len(dataframe)), desc=" File onboarding progress", position=2, leave=False):
            row = dataframe.iloc[j]
            try:
                text_path = row[metadata.name]
            except Exception as e:
                message = f'Data Asset Onboarding Error: Column "{metadata.name}" not found in the local file {path}.'
                logging.exception(message)
                raise Exception(message)

            try:
                text = process_text(text_path, metadata.storage_type)
                batch.append(text)
            except Exception as e:
                logging.exception(e)
                raise Exception(e)

            idx += 1
            if ((idx) % batch_size) == 0:
                batch_index = str(len(files) + 1).zfill(8)
                file_name = f"{folder}/{metadata.name}-{batch_index}.csv.gz"

                df = pd.DataFrame({metadata.name: batch})
                start, end = idx - len(batch), idx
                df["@INDEX"] = range(start, end)
                df.to_csv(file_name, compression="gzip", index=False)
                s3_link = upload_data(file_name, content_type="text/csv", content_encoding="gzip")
                files.append(File(path=s3_link, extension=FileType.CSV, compression="gzip"))
                # get data column index
                data_column_idx = df.columns.to_list().index(metadata.name)
                batch = []

    if len(batch) > 0:
        batch_index = str(len(files) + 1).zfill(8)
        file_name = f"{folder}/{metadata.name}-{batch_index}.csv.gz"

        df = pd.DataFrame({metadata.name: batch})
        start, end = idx - len(batch), idx
        df["@INDEX"] = range(start, end)
        df.to_csv(file_name, compression="gzip", index=False)
        s3_link = upload_data(file_name, content_type="text/csv", content_encoding="gzip")
        files.append(File(path=s3_link, extension=FileType.CSV, compression="gzip"))
        # get data column index
        data_column_idx = df.columns.to_list().index(metadata.name)
        batch = []
    return files, data_column_idx, idx
